@@ -114,6 +114,8 @@ static void do_clear_margins(struct screen *scr)
 				   sw, dh);
 }
 
+static int font_set(struct kmscon_terminal *term);
+
 static void do_redraw_screen(struct screen *scr)
 {
 	int ret;
@@ -129,6 +131,16 @@ static void do_redraw_screen(struct screen *scr)
 	kmscon_text_render(scr->txt);
 
 	ret = uterm_display_swap(scr->disp, false);
+
+	if (ret == -EAGAIN) {
+		uterm_display_deactivate(scr->disp);
+		ret = uterm_display_activate(scr->disp, NULL);
+		if (!ret)
+			ret = font_set(scr->term);
+		if (!ret)
+			ret = uterm_display_swap(scr->disp, false);
+	}
+
 	if (ret) {
 		log_warning("cannot swap display %p", scr->disp);
 		return;
@@ -400,7 +412,7 @@ static void input_event(struct uterm_input *input,
 {
 	struct kmscon_terminal *term = data;
 
-	if (!term->opened || !term->awake || ev->handled)
+	if (!term->opened || !term->awake || ev->handled || !kmscon_session_get_foreground(term->session))
 		return;
 
 	if (conf_grab_matches(term->conf->grab_scroll_up,
@@ -624,6 +636,9 @@ int kmscon_terminal_register(struct kmscon_session **out,
 	if (ret)
 		goto err_con;
 
+	tsm_vte_set_backspace_sends_delete(term->vte,
+					   BUILD_BACKSPACE_SENDS_DELETE);
+
 	ret = tsm_vte_set_palette(term->vte, term->conf->palette);
 	if (ret)
 		goto err_vte;
