@@ -362,6 +362,11 @@ int uterm_drm_display_swap(struct uterm_display *disp, uint32_t fb,
 		ret = drmModePageFlip(vdrm->fd, ddrm->crtc_id, fb,
 				      DRM_MODE_PAGE_FLIP_EVENT, disp);
 		if (ret) {
+			if (disp->desired_mode != disp->default_mode) {
+				disp->desired_mode = disp->default_mode;
+				log_debug("Unable to page-flip desired mode! Switching to default mode.");
+				return -EAGAIN;
+			}
 			log_error("cannot page-flip on DRM-CRTC (%d): %m",
 				  errno);
 			return -EFAULT;
@@ -585,6 +590,17 @@ int uterm_drm_video_find_crtc(struct uterm_video *video, drmModeRes *res,
 	return -1;
 }
 
+static drmModeCrtc *get_current_crtc(int fd, uint32_t encoder_id)
+{
+	drmModeEncoder *enc = drmModeGetEncoder(fd, encoder_id);
+	if (enc) {
+		int crtc_id = enc->crtc_id;
+		drmModeFreeEncoder(enc);
+		return drmModeGetCrtc(fd, crtc_id);
+	}
+	return NULL;
+}
+
 static void bind_display(struct uterm_video *video, drmModeRes *res,
 			 drmModeConnector *conn)
 {
@@ -592,6 +608,7 @@ static void bind_display(struct uterm_video *video, drmModeRes *res,
 	struct uterm_display *disp;
 	struct uterm_drm_display *ddrm;
 	struct uterm_mode *mode;
+	drmModeCrtc *current_crtc;
 	int ret, i;
 
 	ret = display_new(&disp, vdrm->display_ops);
@@ -599,6 +616,8 @@ static void bind_display(struct uterm_video *video, drmModeRes *res,
 		return;
 	ddrm = disp->data;
 
+	current_crtc = get_current_crtc(vdrm->fd, conn->encoder_id);
+
 	for (i = 0; i < conn->count_modes; ++i) {
 		ret = mode_new(&mode, &uterm_drm_mode_ops);
 		if (ret)
@@ -616,8 +635,32 @@ static void bind_display(struct uterm_video *video, drmModeRes *res,
 		if (!disp->default_mode)
 			disp->default_mode = mode;
 
+		/* Save the original KMS mode for later use */
+		if (current_crtc && memcmp(&conn->modes[i], &current_crtc->mode, sizeof(conn->modes[i])) == 0)
+		 	disp->original_mode = mode;
+
+		if (video->desired_width != 0 &&
+		    video->desired_height != 0 &&
+		    mode->ops->get_width(mode) == video->desired_width &&
+		    mode->ops->get_height(mode) == video->desired_height)
+			disp->desired_mode = mode;
+
 		uterm_mode_unref(mode);
 	}
+	if (current_crtc)
+		drmModeFreeCrtc(current_crtc);
+
+	if (!disp->desired_mode) {
+		disp->desired_mode = disp->default_mode;
+	}
+
+	log_debug("Default mode %dx%d",
+		  disp->default_mode->ops->get_width(disp->default_mode),
+		  disp->default_mode->ops->get_height(disp->default_mode));
+
+	log_debug("Desired mode %dx%d",
+		  disp->desired_mode->ops->get_width(disp->desired_mode),
+		  disp->desired_mode->ops->get_height(disp->desired_mode));
 
 	if (shl_dlist_empty(&disp->modes)) {
 		log_warn("no valid mode for display found");
